local BigDebuffs = LibStub("AceAddon-3.0"):GetAddon("BigDebuffs")
local L = LibStub("AceLocale-3.0"):GetLocale("BigDebuffs")

local SpecialDebuffs = {}
for i = 1, #BigDebuffs.SpecialDebuffs do
	local id = BigDebuffs.SpecialDebuffs[i]
	local name = GetSpellInfo(id)
	SpecialDebuffs[name] = {
		type = "toggle",
		get = function(info) local key = info[#info-2] return BigDebuffs.db.profile[key].specialList[id] end,
		set = function(info, value) local key = info[#info-2] BigDebuffs.db.profile[key].specialList[id] = value end,
		name = name,
		desc = L["Show this debuff if present while BigDebuffs are displayed"],
	}
end

Spells = {}
for spellID, spell in pairs(BigDebuffs.Debuffs) do
	if not spell.parent then
		Spells[spell.type] = Spells[spell.type] or {
			name = L[spell.type],
			type = "group",
			args = {},
		}
		local key = "spell"..spellID
		local raidFrames = spell.type == "cc" or spell.type == "roots"
		Spells[spell.type].args[key] = {
			type = "group",
			get = function(info)
				local name = info[#info]
				return BigDebuffs.db.profile.spells[spellID] and BigDebuffs.db.profile.spells[spellID][name]
			end,
			set = function(info, value)
				local name = info[#info]
				BigDebuffs.db.profile.spells[spellID] = BigDebuffs.db.profile.spells[spellID] or {}
				BigDebuffs.db.profile.spells[spellID][name] = value
			end,
			name = GetSpellInfo(spellID),
			args = {
				visibility = {
					order = 1,
					type = "group",
					name = L["Visibility"],
					inline = true,
					get = function(info)
						local name = info[#info]
						local value = (BigDebuffs.db.profile.spells[spellID] and BigDebuffs.db.profile.spells[spellID][name]) or
							(not BigDebuffs.Debuffs[spellID]["no"..name] and 1)
						return value and value == 1
					end,
					set = function(info, value)
						local name = info[#info]
						BigDebuffs.db.profile.spells[spellID] = BigDebuffs.db.profile.spells[spellID] or {}
						value = value and 1 or 0
						BigDebuffs.db.profile.spells[spellID][name] = value

						-- unset if default visibility
						local no = BigDebuffs.Debuffs[spellID]["no"..name]
						if (value == 1 and not no) or
							(value == 0 and no) then
							BigDebuffs.db.profile.spells[spellID][name] = nil
						end
					end,
					args = {
						raidFrames = raidFrames and {
							type = "toggle",
							name = L["Raid Frames"],
							desc = L["Show this spell on the raid frames"],
							width = "full",
							order = 1,
						} or nil,
						unitFrames = {
							type = "toggle",
							name = L["Unit Frames"],
							desc = L["Show this spell on the unit frames"],
							width = "full",
							order = 2
						},
					},
				},
				customPriority = {
					name = L["Custom Priority"],
					type = "toggle",
					width = "full",
					order = 2,
					set = function(info, value)
						BigDebuffs.db.profile.spells[spellID] = BigDebuffs.db.profile.spells[spellID] or {}
						BigDebuffs.db.profile.spells[spellID].customPriority = value
						if not value then
							BigDebuffs.db.profile.spells[spellID].priority = nil
						end
					end,
				},
				priority = {
					name = L["Priority"],
					desc = L["Higher priority spells will take precedence regardless of duration"],
					type = "range",
					min = 1,
					max = 100,
					step = 1,
					order = 3,
					disabled = function() return not BigDebuffs.db.profile.spells[spellID] or not BigDebuffs.db.profile.spells[spellID].customPriority end,
					get = function(info)
						-- Pull the category priority
						return BigDebuffs.db.profile.spells[spellID] and BigDebuffs.db.profile.spells[spellID].priority and
							BigDebuffs.db.profile.spells[spellID].priority or
							BigDebuffs.db.profile.priority[spell.type]
					end,
				},
				customSize = raidFrames and {
					name = L["Custom Scale"],
					type = "toggle",
					width = "full",
					order = 4,
					set = function(info, value)
						local name = info[#info]
						BigDebuffs.db.profile.spells[spellID] = BigDebuffs.db.profile.spells[spellID] or {}
						BigDebuffs.db.profile.spells[spellID].customSize = value
						if not value then
							BigDebuffs.db.profile.spells[spellID].size = nil
						end
					end,
				} or nil,
				size = raidFrames and {
					type = "range",
					name = L["Scale"],
					desc = L["Set the custom size of this spell"],
					get = function(info)
						-- Pull the category size
						return BigDebuffs.db.profile.spells[spellID] and BigDebuffs.db.profile.spells[spellID].size and
							BigDebuffs.db.profile.spells[spellID].size or
							BigDebuffs.db.profile.raidFrames[string.lower(spell.type)]
					end,
					disabled = function() return not BigDebuffs.db.profile.spells[spellID] or not BigDebuffs.db.profile.spells[spellID].customSize end,
					min = 1,
					max = 100,
					step = 1,
					order = 5,
				} or nil,
			},
		}
	end
end

function BigDebuffs:SetupOptions()
	self.options = {
		name = "BigDebuffs",
		descStyle = "inline",
		type = "group",
		plugins = {},
		childGroups = "tab",
		args = {
			vers = {
				order = 1,
				type = "description",
				name = "|cffffd700"..L["Version"].."|r "..GetAddOnMetadata("BigDebuffs", "Version").."\n",
				cmdHidden = true
			},
			desc = {
				order = 2,
				type = "description",
				name = "|cffffd700 "..L["Author"].."|r Jordon\n",
				cmdHidden = true
			},
			raidFrames = {
				name = L["Raid Frames"],
				type = "group",
				order = 10,
				get = function(info) local name = info[#info] return self.db.profile.raidFrames[name] end,
				set = function(info, value) local name = info[#info] self.db.profile.raidFrames[name] = value self:Refresh() end,
				args = {
					enabled = {
						type = "toggle",
						width = "full",
						name = L["Enabled"],
						desc = L["Enable BigDebuffs on raid frames"],
						order = 1,
					},
					cooldownCount = {
						type = "toggle",
						width = "full",
						name = L["Cooldown Count"],
						desc = L["Allow Blizzard and other addons to display countdown text on the icons"],
						order = 2,
					},
					hideBliz = {
						type = "toggle",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						width = "full",
						name = L["Hide Blizzard Debuffs"],
						desc = L["Hides Blizzard debuffs when BigDebuffs are displayed"],
						order = 3,
					},
					redirectBliz = {
						type = "toggle",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						width = "full",
						name = L["Redirect Blizzard Debuffs"],
						desc = L["Redirects Blizzard debuffs to the BigDebuffs anchor"],
						order = 4,
					},
					increaseBuffs = {
						type = "toggle",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						width = "full",
						name = L["Increase Maximum Buffs"],
						desc = L["Sets the maximum buffs to 6"],
						order = 5,
					},
					maxDebuffs = {
						type = "range",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						name = L["Max Debuffs"],
						desc = L["Set the maximum number of debuffs displayed"],
						min = 1,
						max = 20,
						step = 1,
						order = 10,
					},
					anchor = {
						name = L["Anchor"],
						desc = L["Anchor to attach the BigDebuffs frames"],
						type = "select",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						values = {
							["INNER"] = L["INNER"],
							["LEFT"] = L["LEFT"],
							["RIGHT"] = L["RIGHT"],
						},
						order = 11,
					},
					scale = {
						name = L["Scale"],
						type = "group",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						inline = true,
						order = 20,
						args = {
							dispellable = {
								type = "range",
								name = L["Dispellable CC"],
								desc = L["Set the size of dispellable crowd control debuffs"],
								min = 0,
								max = 100,
								step = 1,
								order = 1,
								get = function(info) local name = info[#info] return self.db.profile.raidFrames.dispellable.cc end,
								set = function(info, value) local name = info[#info] self.db.profile.raidFrames.dispellable.cc = value self:Refresh() end,
							},
							cc = {
								type = "range",
								name = L["Other CC"],
								desc = L["Set the size of crowd control debuffs"],
								min = 0,
								max = 100,
								step = 1,
								order = 2,
							},
							special = {
								type = "range",
								name = L["Special Debuffs"],
								desc = L["Set the size of special debuffs"],
								min = 0,
								max = 100,
								step = 1,
								order = 3,
							},
							dispellableRoots = {
								type = "range",
								name = L["Dispellable Roots"],
								desc = L["Set the size of dispellable roots"],
								min = 0,
								max = 100,
								step = 1,
								order = 4,
								get = function(info) local name = info[#info] return self.db.profile.raidFrames.dispellable.roots end,
								set = function(info, value) local name = info[#info] self.db.profile.raidFrames.dispellable.roots= value self:Refresh() end,
							},
							roots = {
								type = "range",
								name = L["Other Roots"],
								desc = L["Set the size of roots"],
								min = 0,
								max = 100,
								step = 1,
								order = 5,
							},
							default = {
								type = "range",
								name = L["Default"],
								desc = L["Set the default size of debuffs"],
								min = 0,
								max = 100,
								step = 1,
								order = 6,
							},
						},
					},
					special = {
						name = L["Special Debuffs"],
						order = 30,
						type = "group",
						disabled = function() return not self.db.profile.raidFrames.enabled end,
						inline = true,
						args = SpecialDebuffs,
					}	
					
				}
			},
			unitFrames = {
				name = L["Unit Frames"],
				type = "group",
				order = 20,
				get = function(info) local name = info[#info] return self.db.profile.unitFrames[name] end,
				set = function(info, value) local name = info[#info] self.db.profile.unitFrames[name] = value self:Refresh() end,
				args = {
					enabled = {
						type = "toggle",
						width = "full",
						name = L["Enabled"],
						desc = L["Enable BigDebuffs on unit frames"],
						order = 1,
					},
					cooldownCount = {
						type = "toggle",
						width = "full",
						name = L["Cooldown Count"],
						desc = L["Allow Blizzard and other addons to display countdown text on the icons"],
						order = 2,
					},
					tooltips = {
						type = "toggle",
						width = "full",
						name = L["Show Tooltips"],
						desc = L["Show spell information when mousing over the icon"],
						order = 3,
					},
					frames = {
						type = "group",
						disabled = function() return not self.db.profile.unitFrames.enabled end,
						order = 10,
						name = L["Frames"],
						inline = true,
						args = {
							player = {
								type = "toggle",
								width = "full",
								name = L["Player Frame"],
								desc = L["Show BigDebuffs on the player frame"],
								order = 1,
							},
							focus = {
								type = "toggle",
								width = "full",
								name = L["Focus Frame"],
								desc = L["Show BigDebuffs on the focus frame"],
								order = 2,
							},
							target = {
								type = "toggle",
								width = "full",
								name = L["Target Frame"],
								desc = L["Show BigDebuffs on the target frame"],
								order = 3,
							},
							pet = {
								type = "toggle",
								width = "full",
								name = L["Pet Frame"],
								desc = L["Show BigDebuffs on the pet frame"],
								order = 4,
							},
							party = {
								type = "toggle",
								width = "full",
								name = L["Party Frames"],
								desc = L["Show BigDebuffs on the party frames"],
								order = 5,
							},
							arena = {
								type = "toggle",
								width = "full",
								name = L["Arena Frames"],
								desc = L["Show BigDebuffs on the arena frames"],
								order = 6,
							},
						},
					},
					spells = {
						order = 20,
						name = L["Spells"],
						type = "group",
						disabled = function() return not self.db.profile.unitFrames.enabled end,
						inline = true,
						args = {
							cc = {
								type = "toggle",
								width = "full",
								name = L["cc"],
								desc = L["Show Crowd Control on the unit frames"],
								order = 1,
							},
							immunities = {
								type = "toggle",
								width = "full",
								name = L["immunities"],
								desc = L["Show Immunities on the unit frames"],
								order = 2,
							},
							buffs = {
								type = "toggle",
								width = "full",
								name = L["buffs"],
								desc = L["Show Buffs on the unit frames"],
								order = 3,
							},
							roots = {
								type = "toggle",
								width = "full",
								name = L["roots"],
								desc = L["Show Roots on the unit frames"],
								order = 4,
							},
						},
					},
				}
			},
			spells = {
				name = L["Spells"],
				type = "group",
				order = 30,
				args = Spells,
			},
		}
	}

	self.options.args.spells.args.priority = {
		name = L["Priority"],
		type = "group",
		get = function(info) local name = info[#info] return self.db.profile.priority[name] end,
		set = function(info, value) local name = info[#info] self.db.profile.priority[name] = value self:Refresh() end,
		inline = true,
		order = 1,
		args = {
			cc = {
				type = "range",
				name = L["cc"],
				desc = L["Higher priority spells will take precedence regardless of duration"],
				min = 1,
				max = 100,
				step = 1,
				order = 1,
			},
			immunities = {
				type = "range",
				name = L["immunities"],
				desc = L["Higher priority spells will take precedence regardless of duration"],
				min = 1,
				max = 100,
				step = 1,
				order = 2,
			},
			buffs = {
				type = "range",
				name = L["buffs"],
				desc = L["Higher priority spells will take precedence regardless of duration"],
				min = 1,
				max = 100,
				step = 1,
				order = 3,
			},
			roots = {
				type = "range",
				name = L["roots"],
				desc = L["Higher priority spells will take precedence regardless of duration"],
				min = 1,
				max = 100,
				step = 1,
				order = 4,
			},
		},
	}

	self.options.plugins.profiles = { profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db) }
	
	local LibDualSpec = LibStub('LibDualSpec-1.0')
	LibDualSpec:EnhanceDatabase(self.db, "BigDebuffsDB")
	LibDualSpec:EnhanceOptions(self.options.plugins.profiles.profiles, self.db)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("BigDebuffs", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BigDebuffs", "BigDebuffs")
end
